/****************************************************************************
** MPBoard meta object code from reading C++ file 'mp_board.h'
**
** Created: Tue Dec 26 17:46:00 2017
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#undef QT_NO_COMPAT
#include "../../../src/kdegames-3.5.10/libksirtet/lib/mp_board.h"
#include <qmetaobject.h>
#include <qapplication.h>

#include <private/qucomextra_p.h>
#if !defined(Q_MOC_OUTPUT_REVISION) || (Q_MOC_OUTPUT_REVISION != 26)
#error "This file was generated using the moc from 3.3.8b. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

const char *MPBoard::className() const
{
    return "MPBoard";
}

QMetaObject *MPBoard::metaObj = 0;
static QMetaObjectCleanUp cleanUp_MPBoard( "MPBoard", &MPBoard::staticMetaObject );

#ifndef QT_NO_TRANSLATION
QString MPBoard::tr( const char *s, const char *c )
{
    if ( qApp )
	return qApp->translate( "MPBoard", s, c, QApplication::DefaultCodec );
    else
	return QString::fromLatin1( s );
}
#ifndef QT_NO_TRANSLATION_UTF8
QString MPBoard::trUtf8( const char *s, const char *c )
{
    if ( qApp )
	return qApp->translate( "MPBoard", s, c, QApplication::UnicodeUTF8 );
    else
	return QString::fromUtf8( s );
}
#endif // QT_NO_TRANSLATION_UTF8

#endif // QT_NO_TRANSLATION

QMetaObject* MPBoard::staticMetaObject()
{
    if ( metaObj )
	return metaObj;
    QMetaObject* parentObject = QWidget::staticMetaObject();
    static const QUParameter param_signal_0[] = {
	{ 0, &static_QUType_bool, 0, QUParameter::In }
    };
    static const QUMethod signal_0 = {"enableKeys", 1, param_signal_0 };
    static const QMetaData signal_tbl[] = {
	{ "enableKeys(bool)", &signal_0, QMetaData::Public }
    };
    metaObj = QMetaObject::new_metaobject(
	"MPBoard", parentObject,
	0, 0,
	signal_tbl, 1,
#ifndef QT_NO_PROPERTIES
	0, 0,
	0, 0,
#endif // QT_NO_PROPERTIES
	0, 0 );
    cleanUp_MPBoard.setMetaObject( metaObj );
    return metaObj;
}

void* MPBoard::qt_cast( const char* clname )
{
    if ( !qstrcmp( clname, "MPBoard" ) )
	return this;
    return QWidget::qt_cast( clname );
}

// SIGNAL enableKeys
void MPBoard::enableKeys( bool t0 )
{
    activate_signal_bool( staticMetaObject()->signalOffset() + 0, t0 );
}

bool MPBoard::qt_invoke( int _id, QUObject* _o )
{
    return QWidget::qt_invoke(_id,_o);
}

bool MPBoard::qt_emit( int _id, QUObject* _o )
{
    switch ( _id - staticMetaObject()->signalOffset() ) {
    case 0: enableKeys((bool)static_QUType_bool.get(_o+1)); break;
    default:
	return QWidget::qt_emit(_id,_o);
    }
    return TRUE;
}
#ifndef QT_NO_PROPERTIES

bool MPBoard::qt_property( int id, int f, QVariant* v)
{
    return QWidget::qt_property( id, f, v);
}

bool MPBoard::qt_static_property( QObject* , int , int , QVariant* ){ return FALSE; }
#endif // QT_NO_PROPERTIES
