#include <kdialog.h>
#include <klocale.h>
/****************************************************************************
** Form implementation generated from reading ui file '/usr/src/ports/kdegames3/kdegames3-3.5.10-11.x86_64/src/kdegames-3.5.10/ksnake/appearance.ui'
**
** Created by User Interface Compiler
**
** WARNING! All changes made in this file will be lost!
****************************************************************************/

#include "appearance.h"

#include <qvariant.h>
#include <qbuttongroup.h>
#include <qcombobox.h>
#include <kcolorbutton.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include "kcolorbutton.h"

/*
 *  Constructs a Appearance as a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'.
 */
Appearance::Appearance( QWidget* parent, const char* name, WFlags fl )
    : QWidget( parent, name, fl )
{
    if ( !name )
	setName( "Appearance" );
    AppearanceLayout = new QVBoxLayout( this, 11, 6, "AppearanceLayout"); 

    buttonGroup1 = new QButtonGroup( this, "buttonGroup1" );
    buttonGroup1->setColumnLayout(0, Qt::Vertical );
    buttonGroup1->layout()->setSpacing( 6 );
    buttonGroup1->layout()->setMargin( 11 );
    buttonGroup1Layout = new QGridLayout( buttonGroup1->layout() );
    buttonGroup1Layout->setAlignment( Qt::AlignTop );

    kcfg_bgimage = new QComboBox( FALSE, buttonGroup1, "kcfg_bgimage" );

    buttonGroup1Layout->addWidget( kcfg_bgimage, 1, 1 );

    kcfg_bgcolor = new KColorButton( buttonGroup1, "kcfg_bgcolor" );

    buttonGroup1Layout->addWidget( kcfg_bgcolor, 0, 1 );

    kcfg_bgcolor_enabled = new QRadioButton( buttonGroup1, "kcfg_bgcolor_enabled" );
    kcfg_bgcolor_enabled->setChecked( TRUE );

    buttonGroup1Layout->addWidget( kcfg_bgcolor_enabled, 0, 0 );

    kcfg_bgimage_enabled = new QRadioButton( buttonGroup1, "kcfg_bgimage_enabled" );

    buttonGroup1Layout->addWidget( kcfg_bgimage_enabled, 1, 0 );
    AppearanceLayout->addWidget( buttonGroup1 );
    spacer2 = new QSpacerItem( 21, 40, QSizePolicy::Minimum, QSizePolicy::Expanding );
    AppearanceLayout->addItem( spacer2 );
    languageChange();
    resize( QSize(334, 285).expandedTo(minimumSizeHint()) );
    clearWState( WState_Polished );
}

/*
 *  Destroys the object and frees any allocated resources
 */
Appearance::~Appearance()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void Appearance::languageChange()
{
    buttonGroup1->setTitle( tr2i18n( "Background" ) );
    kcfg_bgcolor->setText( QString::null );
    kcfg_bgcolor_enabled->setText( tr2i18n( "Color:" ) );
    kcfg_bgimage_enabled->setText( tr2i18n( "Image:" ) );
}

#include "appearance.moc"
