/****************************************************************************
** Form interface generated from reading ui file '/usr/src/ports/kdegames3/kdegames3-3.5.10-11.x86_64/src/kdegames-3.5.10/ksnake/appearance.ui'
**
** Created by User Interface Compiler
**
** WARNING! All changes made in this file will be lost!
****************************************************************************/

#ifndef APPEARANCE_H
#define APPEARANCE_H

#include <qvariant.h>
#include <qwidget.h>

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QSpacerItem;
class QButtonGroup;
class QComboBox;
class KColorButton;
class QRadioButton;

class Appearance : public QWidget
{
    Q_OBJECT

public:
    Appearance( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~Appearance();

    QButtonGroup* buttonGroup1;
    QComboBox* kcfg_bgimage;
    KColorButton* kcfg_bgcolor;
    QRadioButton* kcfg_bgcolor_enabled;
    QRadioButton* kcfg_bgimage_enabled;

protected:
    QVBoxLayout* AppearanceLayout;
    QSpacerItem* spacer2;
    QGridLayout* buttonGroup1Layout;

protected slots:
    virtual void languageChange();

};

#endif // APPEARANCE_H
