/****************************************************************************
** Form interface generated from reading ui file '/usr/src/ports/kdegames3/kdegames3-3.5.10-11.x86_64/src/kdegames-3.5.10/ksnake/general.ui'
**
** Created by User Interface Compiler
**
** WARNING! All changes made in this file will be lost!
****************************************************************************/

#ifndef GENERAL_H
#define GENERAL_H

#include <qvariant.h>
#include <qwidget.h>

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QSpacerItem;
class QGroupBox;
class QSlider;
class QLabel;
class QComboBox;
class QSpinBox;

class General : public QWidget
{
    Q_OBJECT

public:
    General( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~General();

    QGroupBox* groupBox4;
    QSlider* kcfg_Skill;
    QLabel* textLabel10;
    QLabel* textLabel12;
    QGroupBox* groupBox6;
    QLabel* textLabel7;
    QComboBox* kcfg_SnakesAI;
    QSpinBox* kcfg_ComputerSnakes;
    QLabel* textLabel8;
    QGroupBox* groupBox5;
    QLabel* textLabel9;
    QSpinBox* kcfg_Balls;
    QComboBox* kcfg_BallsAI;
    QLabel* textLabel13;

protected:
    QVBoxLayout* GeneralLayout;
    QSpacerItem* spacer1;
    QGridLayout* groupBox4Layout;
    QGridLayout* groupBox6Layout;
    QGridLayout* groupBox5Layout;

protected slots:
    virtual void languageChange();

};

#endif // GENERAL_H
