#include <kdialog.h>
#include <klocale.h>
/****************************************************************************
** Form implementation generated from reading ui file '/usr/src/ports/kdegames3/kdegames3-3.5.10-11.x86_64/src/kdegames-3.5.10/kenolaba/EvalDlg.ui'
**
** Created by User Interface Compiler
**
** WARNING! All changes made in this file will be lost!
****************************************************************************/

#include "EvalDlg.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qtabwidget.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qlcdnumber.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>

static const char* const img0_EvalDlg[] = { 
"16 16 3 1",
". c None",
"# c #838183",
"a c #ffffff",
".....#####......",
"...#########....",
"..###aaaaa###...",
".###a.....a###a.",
".##a........##a.",
"##a..........##a",
"##a..........##a",
"##a..........##a",
"##a..........##a",
"##a..........##a",
".##a........##a.",
".###.......###a.",
"..###.....###a..",
"...#########a...",
"...aa#####aa....",
".....aaaaa......"};

static const char* const img1_EvalDlg[] = { 
"16 16 6 1",
". c None",
"# c #000000",
"a c #410000",
"b c #830000",
"c c #ff0000",
"d c #ffffff",
".....#####......",
"...##aaaaa##....",
"..##aabbbbb##...",
".##aabbbbbbb##..",
".#aabbbbbbbbb#..",
"#aabbbbbbbbbbb#.",
"#aabbbbbcccbbb#.",
"#aabbbbcccccbb#.",
"#aabbbcccccccb#.",
"#aabbbccccdccb#.",
".#aabbcccdddc#..",
".##aabbcccdc##..",
"..##aabbccc##...",
"...##aaaaa##....",
".....#####......",
"................"};

static const char* const img2_EvalDlg[] = { 
"16 16 6 1",
". c None",
"# c #000000",
"a c #838100",
"b c #c5c600",
"c c #ffff00",
"d c #ffffff",
".....#####......",
"...##aaaaa##....",
"..##aaaaaaa##...",
".##aaaabbbaa##..",
".#aaaabbbbbaa#..",
"#aaaabbbbbbbaa#.",
"#aaabbbbbbbbba#.",
"#aaabbbccccbba#.",
"#aaabbbcccccba#.",
"#aaaabbcccdcaa#.",
".#aaaabccddda#..",
".##aaaabccdc##..",
"..##aaaaaaa##...",
"...##aaaaa##....",
".....#####......",
"................"};


/*
 *  Constructs a EvalDlg as a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'.
 */
EvalDlg::EvalDlg( QWidget* parent, const char* name, WFlags fl )
    : QWidget( parent, name, fl ),
      image0( (const char **) img0_EvalDlg ),
      image1( (const char **) img1_EvalDlg ),
      image2( (const char **) img2_EvalDlg )
{
    if ( !name )
	setName( "EvalDlg" );
    EvalDlgLayout = new QVBoxLayout( this, 11, 6, "EvalDlgLayout"); 

    TabWidget2 = new QTabWidget( this, "TabWidget2" );

    moves = new QWidget( TabWidget2, "moves" );
    movesLayout = new QVBoxLayout( moves, 11, 6, "movesLayout"); 

    Layout17 = new QGridLayout( 0, 1, 1, 0, 6, "Layout17"); 

    Layout35 = new QGridLayout( 0, 1, 1, 0, 4, "Layout35"); 

    moveEval1 = new QLineEdit( moves, "moveEval1" );
    moveEval1->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, moveEval1->sizePolicy().hasHeightForWidth() ) );
    moveEval1->setMaximumSize( QSize( 30, 32767 ) );
    moveEval1->setAlignment( int( QLineEdit::AlignLeft ) );

    Layout35->addWidget( moveEval1, 0, 5 );

    PixmapLabel3_2 = new QLabel( moves, "PixmapLabel3_2" );
    PixmapLabel3_2->setPixmap( image0 );
    PixmapLabel3_2->setScaledContents( FALSE );

    Layout35->addWidget( PixmapLabel3_2, 1, 4 );

    moveEval2 = new QLineEdit( moves, "moveEval2" );
    moveEval2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, moveEval2->sizePolicy().hasHeightForWidth() ) );
    moveEval2->setMaximumSize( QSize( 30, 32767 ) );
    moveEval2->setAlignment( int( QLineEdit::AlignLeft ) );

    Layout35->addWidget( moveEval2, 1, 5 );

    PixmapLabel1_2_5 = new QLabel( moves, "PixmapLabel1_2_5" );
    PixmapLabel1_2_5->setPixmap( image1 );
    PixmapLabel1_2_5->setScaledContents( FALSE );

    Layout35->addWidget( PixmapLabel1_2_5, 2, 2 );

    moveEval3 = new QLineEdit( moves, "moveEval3" );
    moveEval3->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, moveEval3->sizePolicy().hasHeightForWidth() ) );
    moveEval3->setMaximumSize( QSize( 30, 32767 ) );
    moveEval3->setAlignment( int( QLineEdit::AlignLeft ) );

    Layout35->addWidget( moveEval3, 2, 5 );

    PixmapLabel3_3 = new QLabel( moves, "PixmapLabel3_3" );
    PixmapLabel3_3->setPixmap( image0 );
    PixmapLabel3_3->setScaledContents( FALSE );

    Layout35->addWidget( PixmapLabel3_3, 2, 4 );

    unnamed = new QLabel( moves, "unnamed" );
    unnamed->setPixmap( image1 );
    unnamed->setScaledContents( FALSE );

    Layout35->addWidget( unnamed, 0, 3 );

    PixmapLabel1_2_6 = new QLabel( moves, "PixmapLabel1_2_6" );
    PixmapLabel1_2_6->setPixmap( image1 );
    PixmapLabel1_2_6->setScaledContents( FALSE );

    Layout35->addWidget( PixmapLabel1_2_6, 2, 3 );

    PixmapLabel1_2_4 = new QLabel( moves, "PixmapLabel1_2_4" );
    PixmapLabel1_2_4->setPixmap( image1 );
    PixmapLabel1_2_4->setScaledContents( FALSE );

    Layout35->addWidget( PixmapLabel1_2_4, 1, 3 );

    PixmapLabel3 = new QLabel( moves, "PixmapLabel3" );
    PixmapLabel3->setPixmap( image0 );
    PixmapLabel3->setScaledContents( FALSE );

    Layout35->addWidget( PixmapLabel3, 0, 4 );

    PixmapLabel1_2 = new QLabel( moves, "PixmapLabel1_2" );
    PixmapLabel1_2->setPixmap( image1 );
    PixmapLabel1_2->setScaledContents( FALSE );

    Layout35->addWidget( PixmapLabel1_2, 1, 2 );

    PixmapLabel1_3 = new QLabel( moves, "PixmapLabel1_3" );
    PixmapLabel1_3->setPixmap( image1 );
    PixmapLabel1_3->setScaledContents( FALSE );

    Layout35->addWidget( PixmapLabel1_3, 2, 1 );
    Spacer2 = new QSpacerItem( 0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
    Layout35->addItem( Spacer2, 1, 0 );

    Layout17->addLayout( Layout35, 0, 2 );

    TextLabel4_2_2 = new QLabel( moves, "TextLabel4_2_2" );
    TextLabel4_2_2->setAlignment( int( QLabel::AlignTop | QLabel::AlignLeft ) );

    Layout17->addWidget( TextLabel4_2_2, 2, 1 );
    Spacer9_2 = new QSpacerItem( 0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
    Layout17->addItem( Spacer9_2, 1, 0 );

    TextLabel4_2 = new QLabel( moves, "TextLabel4_2" );
    TextLabel4_2->setAlignment( int( QLabel::AlignTop | QLabel::AlignLeft ) );

    Layout17->addWidget( TextLabel4_2, 1, 1 );

    Layout37 = new QGridLayout( 0, 1, 1, 0, 4, "Layout37"); 

    PixmapLabel1_2_5_2_3 = new QLabel( moves, "PixmapLabel1_2_5_2_3" );
    PixmapLabel1_2_5_2_3->setPixmap( image1 );
    PixmapLabel1_2_5_2_3->setScaledContents( FALSE );

    Layout37->addWidget( PixmapLabel1_2_5_2_3, 2, 3 );

    PixmapLabel3_3_2_3 = new QLabel( moves, "PixmapLabel3_3_2_3" );
    PixmapLabel3_3_2_3->setPixmap( image0 );
    PixmapLabel3_3_2_3->setScaledContents( FALSE );

    Layout37->addWidget( PixmapLabel3_3_2_3, 2, 6 );

    PixmapLabel3_3_2_2 = new QLabel( moves, "PixmapLabel3_3_2_2" );
    PixmapLabel3_3_2_2->setPixmap( image0 );
    PixmapLabel3_3_2_2->setScaledContents( FALSE );

    Layout37->addWidget( PixmapLabel3_3_2_2, 1, 6 );

    moveEval4 = new QLineEdit( moves, "moveEval4" );
    moveEval4->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, moveEval4->sizePolicy().hasHeightForWidth() ) );
    moveEval4->setMaximumSize( QSize( 30, 32767 ) );
    moveEval4->setAlignment( int( QLineEdit::AlignLeft ) );

    Layout37->addWidget( moveEval4, 0, 7 );

    moveEval5 = new QLineEdit( moves, "moveEval5" );
    moveEval5->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, moveEval5->sizePolicy().hasHeightForWidth() ) );
    moveEval5->setMaximumSize( QSize( 30, 32767 ) );
    moveEval5->setAlignment( int( QLineEdit::AlignLeft ) );

    Layout37->addWidget( moveEval5, 1, 7 );

    PixmapLabel2_3 = new QLabel( moves, "PixmapLabel2_3" );
    PixmapLabel2_3->setPixmap( image2 );
    PixmapLabel2_3->setScaledContents( FALSE );

    Layout37->addWidget( PixmapLabel2_3, 2, 5 );

    PixmapLabel1_2_5_2 = new QLabel( moves, "PixmapLabel1_2_5_2" );
    PixmapLabel1_2_5_2->setPixmap( image1 );
    PixmapLabel1_2_5_2->setScaledContents( FALSE );

    Layout37->addWidget( PixmapLabel1_2_5_2, 0, 4 );

    PixmapLabel3_3_2 = new QLabel( moves, "PixmapLabel3_3_2" );
    PixmapLabel3_3_2->setPixmap( image0 );
    PixmapLabel3_3_2->setScaledContents( FALSE );

    Layout37->addWidget( PixmapLabel3_3_2, 0, 6 );

    PixmapLabel2_2_2 = new QLabel( moves, "PixmapLabel2_2_2" );
    PixmapLabel2_2_2->setPixmap( image2 );
    PixmapLabel2_2_2->setScaledContents( FALSE );

    Layout37->addWidget( PixmapLabel2_2_2, 2, 4 );

    PixmapLabel1_3_2 = new QLabel( moves, "PixmapLabel1_3_2" );
    PixmapLabel1_3_2->setPixmap( image1 );
    PixmapLabel1_3_2->setScaledContents( FALSE );

    Layout37->addWidget( PixmapLabel1_3_2, 2, 1 );
    Spacer2_2 = new QSpacerItem( 0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
    Layout37->addItem( Spacer2_2, 1, 0 );

    moveEval6 = new QLineEdit( moves, "moveEval6" );
    moveEval6->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, moveEval6->sizePolicy().hasHeightForWidth() ) );
    moveEval6->setMaximumSize( QSize( 30, 32767 ) );
    moveEval6->setAlignment( int( QLineEdit::AlignLeft ) );

    Layout37->addWidget( moveEval6, 2, 7 );

    PixmapLabel1_4 = new QLabel( moves, "PixmapLabel1_4" );
    PixmapLabel1_4->setPixmap( image1 );
    PixmapLabel1_4->setScaledContents( FALSE );

    Layout37->addWidget( PixmapLabel1_4, 0, 3 );

    PixmapLabel1_2_2 = new QLabel( moves, "PixmapLabel1_2_2" );
    PixmapLabel1_2_2->setPixmap( image1 );
    PixmapLabel1_2_2->setScaledContents( FALSE );

    Layout37->addWidget( PixmapLabel1_2_2, 1, 2 );

    PixmapLabel2_2 = new QLabel( moves, "PixmapLabel2_2" );
    PixmapLabel2_2->setPixmap( image2 );
    PixmapLabel2_2->setScaledContents( FALSE );

    Layout37->addWidget( PixmapLabel2_2, 1, 5 );

    PixmapLabel1_4_3 = new QLabel( moves, "PixmapLabel1_4_3" );
    PixmapLabel1_4_3->setPixmap( image1 );
    PixmapLabel1_4_3->setScaledContents( FALSE );

    Layout37->addWidget( PixmapLabel1_4_3, 2, 2 );

    PixmapLabel1_2_5_2_2 = new QLabel( moves, "PixmapLabel1_2_5_2_2" );
    PixmapLabel1_2_5_2_2->setPixmap( image1 );

    Layout37->addWidget( PixmapLabel1_2_5_2_2, 1, 4 );

    PixmapLabel2 = new QLabel( moves, "PixmapLabel2" );
    PixmapLabel2->setPixmap( image2 );
    PixmapLabel2->setScaledContents( FALSE );

    Layout37->addWidget( PixmapLabel2, 0, 5 );

    PixmapLabel1_4_2 = new QLabel( moves, "PixmapLabel1_4_2" );
    PixmapLabel1_4_2->setPixmap( image1 );
    PixmapLabel1_4_2->setScaledContents( FALSE );

    Layout37->addWidget( PixmapLabel1_4_2, 1, 3 );

    Layout17->addLayout( Layout37, 1, 2 );
    Spacer9 = new QSpacerItem( 0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
    Layout17->addItem( Spacer9, 1, 3 );

    Layout38 = new QGridLayout( 0, 1, 1, 0, 4, "Layout38"); 

    PixmapLabel2_4 = new QLabel( moves, "PixmapLabel2_4" );
    PixmapLabel2_4->setPixmap( image2 );
    PixmapLabel2_4->setScaledContents( FALSE );

    Layout38->addWidget( PixmapLabel2_4, 0, 5 );

    PixmapLabel1_3_2_2 = new QLabel( moves, "PixmapLabel1_3_2_2" );
    PixmapLabel1_3_2_2->setPixmap( image1 );
    PixmapLabel1_3_2_2->setScaledContents( FALSE );

    Layout38->addWidget( PixmapLabel1_3_2_2, 2, 1 );

    PixmapLabel1_2_5_2_2_2 = new QLabel( moves, "PixmapLabel1_2_5_2_2_2" );
    PixmapLabel1_2_5_2_2_2->setPixmap( image1 );

    Layout38->addWidget( PixmapLabel1_2_5_2_2_2, 1, 4 );
    Spacer2_2_2 = new QSpacerItem( 0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
    Layout38->addItem( Spacer2_2_2, 1, 0 );

    moveEval7 = new QLineEdit( moves, "moveEval7" );
    moveEval7->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, moveEval7->sizePolicy().hasHeightForWidth() ) );
    moveEval7->setMaximumSize( QSize( 30, 32767 ) );
    moveEval7->setAlignment( int( QLineEdit::AlignLeft ) );

    Layout38->addWidget( moveEval7, 0, 7 );

    PixmapLabel1_2_2_2 = new QLabel( moves, "PixmapLabel1_2_2_2" );
    PixmapLabel1_2_2_2->setPixmap( image1 );
    PixmapLabel1_2_2_2->setScaledContents( FALSE );

    Layout38->addWidget( PixmapLabel1_2_2_2, 1, 2 );

    PixmapLabel2_2_2_3 = new QLabel( moves, "PixmapLabel2_2_2_3" );
    PixmapLabel2_2_2_3->setPixmap( image2 );
    PixmapLabel2_2_2_3->setScaledContents( FALSE );

    Layout38->addWidget( PixmapLabel2_2_2_3, 2, 4 );

    moveEval9 = new QLineEdit( moves, "moveEval9" );
    moveEval9->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, moveEval9->sizePolicy().hasHeightForWidth() ) );
    moveEval9->setMaximumSize( QSize( 30, 32767 ) );
    moveEval9->setAlignment( int( QLineEdit::AlignLeft ) );

    Layout38->addWidget( moveEval9, 2, 7 );

    PixmapLabel1_4_2_2 = new QLabel( moves, "PixmapLabel1_4_2_2" );
    PixmapLabel1_4_2_2->setPixmap( image1 );
    PixmapLabel1_4_2_2->setScaledContents( FALSE );

    Layout38->addWidget( PixmapLabel1_4_2_2, 1, 3 );

    PixmapLabel1_2_5_2_3_3 = new QLabel( moves, "PixmapLabel1_2_5_2_3_3" );
    PixmapLabel1_2_5_2_3_3->setPixmap( image1 );
    PixmapLabel1_2_5_2_3_3->setScaledContents( FALSE );

    Layout38->addWidget( PixmapLabel1_2_5_2_3_3, 2, 3 );

    moveEval8 = new QLineEdit( moves, "moveEval8" );
    moveEval8->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, moveEval8->sizePolicy().hasHeightForWidth() ) );
    moveEval8->setMaximumSize( QSize( 30, 32767 ) );
    moveEval8->setAlignment( int( QLineEdit::AlignLeft ) );

    Layout38->addWidget( moveEval8, 1, 7 );

    PixmapLabel1_4_3_2 = new QLabel( moves, "PixmapLabel1_4_3_2" );
    PixmapLabel1_4_3_2->setPixmap( image1 );
    PixmapLabel1_4_3_2->setScaledContents( FALSE );

    Layout38->addWidget( PixmapLabel1_4_3_2, 2, 2 );

    PixmapLabel1_4_4 = new QLabel( moves, "PixmapLabel1_4_4" );
    PixmapLabel1_4_4->setPixmap( image1 );
    PixmapLabel1_4_4->setScaledContents( FALSE );

    Layout38->addWidget( PixmapLabel1_4_4, 0, 3 );

    PixmapLabel1_2_5_2_4 = new QLabel( moves, "PixmapLabel1_2_5_2_4" );
    PixmapLabel1_2_5_2_4->setPixmap( image1 );
    PixmapLabel1_2_5_2_4->setScaledContents( FALSE );

    Layout38->addWidget( PixmapLabel1_2_5_2_4, 0, 4 );

    PixmapLabel2_3_3 = new QLabel( moves, "PixmapLabel2_3_3" );
    PixmapLabel2_3_3->setPixmap( image2 );
    PixmapLabel2_3_3->setScaledContents( FALSE );

    Layout38->addWidget( PixmapLabel2_3_3, 2, 5 );

    PixmapLabel2_2_3 = new QLabel( moves, "PixmapLabel2_2_3" );
    PixmapLabel2_2_3->setPixmap( image2 );
    PixmapLabel2_2_3->setScaledContents( FALSE );

    Layout38->addWidget( PixmapLabel2_2_3, 1, 5 );

    Layout17->addLayout( Layout38, 2, 2 );

    TextLabel4 = new QLabel( moves, "TextLabel4" );
    TextLabel4->setAlignment( int( QLabel::AlignTop | QLabel::AlignLeft ) );

    Layout17->addWidget( TextLabel4, 0, 1 );
    movesLayout->addLayout( Layout17 );

    TextLabel5 = new QLabel( moves, "TextLabel5" );
    TextLabel5->setMinimumSize( QSize( 0, 20 ) );
    TextLabel5->setAlignment( int( QLabel::WordBreak | QLabel::AlignTop | QLabel::AlignLeft ) );
    movesLayout->addWidget( TextLabel5 );
    Spacer5 = new QSpacerItem( 0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding );
    movesLayout->addItem( Spacer5 );
    TabWidget2->insertTab( moves, QString::fromLatin1("") );

    position = new QWidget( TabWidget2, "position" );
    positionLayout = new QVBoxLayout( position, 11, 6, "positionLayout"); 

    Layout63 = new QHBoxLayout( 0, 0, 0, "Layout63"); 
    Spacer12 = new QSpacerItem( 0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
    Layout63->addItem( Spacer12 );

    Layout62 = new QGridLayout( 0, 1, 1, 0, 6, "Layout62"); 

    TextLabel1_2_3_2 = new QLabel( position, "TextLabel1_2_3_2" );

    Layout62->addWidget( TextLabel1_2_3_2, 3, 0 );

    posEval3 = new QLineEdit( position, "posEval3" );
    posEval3->setMaximumSize( QSize( 30, 32767 ) );
    posEval3->setAlignment( int( QLineEdit::AlignLeft ) );

    Layout62->addWidget( posEval3, 2, 1 );

    posEval5 = new QLineEdit( position, "posEval5" );
    posEval5->setMaximumSize( QSize( 30, 32767 ) );
    posEval5->setAlignment( int( QLineEdit::AlignLeft ) );

    Layout62->addWidget( posEval5, 4, 1 );

    posEval2 = new QLineEdit( position, "posEval2" );
    posEval2->setMaximumSize( QSize( 30, 32767 ) );
    posEval2->setAlignment( int( QLineEdit::AlignLeft ) );

    Layout62->addWidget( posEval2, 1, 1 );

    TextLabel1_2 = new QLabel( position, "TextLabel1_2" );

    Layout62->addWidget( TextLabel1_2, 4, 0 );

    diffEval2 = new QLineEdit( position, "diffEval2" );
    diffEval2->setMaximumSize( QSize( 30, 32767 ) );
    diffEval2->setAlignment( int( QLineEdit::AlignLeft ) );

    Layout62->addWidget( diffEval2, 1, 3 );

    diffEval3 = new QLineEdit( position, "diffEval3" );
    diffEval3->setMaximumSize( QSize( 30, 32767 ) );
    diffEval3->setAlignment( int( QLineEdit::AlignLeft ) );

    Layout62->addWidget( diffEval3, 2, 3 );

    posEval4 = new QLineEdit( position, "posEval4" );
    posEval4->setMaximumSize( QSize( 30, 32767 ) );
    posEval4->setAlignment( int( QLineEdit::AlignLeft ) );

    Layout62->addWidget( posEval4, 3, 1 );

    TextLabel1_2_2 = new QLabel( position, "TextLabel1_2_2" );

    Layout62->addWidget( TextLabel1_2_2, 0, 0 );

    diffEval5 = new QLineEdit( position, "diffEval5" );
    diffEval5->setMaximumSize( QSize( 30, 32767 ) );
    diffEval5->setAlignment( int( QLineEdit::AlignLeft ) );

    Layout62->addWidget( diffEval5, 4, 3 );

    posEval1 = new QLineEdit( position, "posEval1" );
    posEval1->setMaximumSize( QSize( 30, 32767 ) );
    posEval1->setAlignment( int( QLineEdit::AlignLeft ) );

    Layout62->addWidget( posEval1, 0, 1 );

    TextLabel1_2_3 = new QLabel( position, "TextLabel1_2_3" );

    Layout62->addWidget( TextLabel1_2_3, 2, 0 );

    TextLabel3_4 = new QLabel( position, "TextLabel3_4" );

    Layout62->addWidget( TextLabel3_4, 2, 2 );

    TextLabel3_3 = new QLabel( position, "TextLabel3_3" );

    Layout62->addWidget( TextLabel3_3, 3, 2 );

    diffEval4 = new QLineEdit( position, "diffEval4" );
    diffEval4->setMaximumSize( QSize( 30, 32767 ) );
    diffEval4->setAlignment( int( QLineEdit::AlignLeft ) );

    Layout62->addWidget( diffEval4, 3, 3 );

    TextLabel3_2 = new QLabel( position, "TextLabel3_2" );

    Layout62->addWidget( TextLabel3_2, 4, 2 );

    TextLabel1_2_2_2 = new QLabel( position, "TextLabel1_2_2_2" );

    Layout62->addWidget( TextLabel1_2_2_2, 1, 0 );

    TextLabel3 = new QLabel( position, "TextLabel3" );

    Layout62->addWidget( TextLabel3, 1, 2 );
    Layout63->addLayout( Layout62 );
    Spacer11 = new QSpacerItem( 0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
    Layout63->addItem( Spacer11 );
    positionLayout->addLayout( Layout63 );

    TextLabel6 = new QLabel( position, "TextLabel6" );
    TextLabel6->setAlignment( int( QLabel::WordBreak | QLabel::AlignTop | QLabel::AlignLeft ) );
    positionLayout->addWidget( TextLabel6 );
    Spacer4 = new QSpacerItem( 0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding );
    positionLayout->addItem( Spacer4 );
    TabWidget2->insertTab( position, QString::fromLatin1("") );

    inarow = new QWidget( TabWidget2, "inarow" );
    inarowLayout = new QVBoxLayout( inarow, 11, 6, "inarowLayout"); 

    Layout64 = new QHBoxLayout( 0, 0, 6, "Layout64"); 
    Spacer14 = new QSpacerItem( 0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
    Layout64->addItem( Spacer14 );

    Layout42 = new QGridLayout( 0, 1, 1, 0, 6, "Layout42"); 

    rowEval3 = new QLineEdit( inarow, "rowEval3" );
    rowEval3->setMaximumSize( QSize( 30, 32767 ) );
    rowEval3->setAlignment( int( QLineEdit::AlignLeft ) );

    Layout42->addWidget( rowEval3, 1, 1 );

    rowEval2 = new QLineEdit( inarow, "rowEval2" );
    rowEval2->setMaximumSize( QSize( 30, 32767 ) );
    rowEval2->setAlignment( int( QLineEdit::AlignLeft ) );

    Layout42->addWidget( rowEval2, 0, 1 );

    rowEval5 = new QLineEdit( inarow, "rowEval5" );
    rowEval5->setMaximumSize( QSize( 30, 32767 ) );
    rowEval5->setAlignment( int( QLineEdit::AlignLeft ) );

    Layout42->addWidget( rowEval5, 3, 1 );

    rowEval4 = new QLineEdit( inarow, "rowEval4" );
    rowEval4->setMaximumSize( QSize( 30, 32767 ) );
    rowEval4->setAlignment( int( QLineEdit::AlignLeft ) );

    Layout42->addWidget( rowEval4, 2, 1 );

    TextLabel1_2_2_3_2 = new QLabel( inarow, "TextLabel1_2_2_3_2" );

    Layout42->addWidget( TextLabel1_2_2_3_2, 1, 0 );

    TextLabel1_2_2_3 = new QLabel( inarow, "TextLabel1_2_2_3" );

    Layout42->addWidget( TextLabel1_2_2_3, 0, 0 );

    TextLabel1_2_2_3_3 = new QLabel( inarow, "TextLabel1_2_2_3_3" );

    Layout42->addWidget( TextLabel1_2_2_3_3, 2, 0 );

    TextLabel1_2_2_3_4 = new QLabel( inarow, "TextLabel1_2_2_3_4" );

    Layout42->addWidget( TextLabel1_2_2_3_4, 3, 0 );
    Layout64->addLayout( Layout42 );
    Spacer13 = new QSpacerItem( 0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
    Layout64->addItem( Spacer13 );
    inarowLayout->addLayout( Layout64 );

    TextLabel6_2 = new QLabel( inarow, "TextLabel6_2" );
    TextLabel6_2->setAlignment( int( QLabel::WordBreak | QLabel::AlignTop | QLabel::AlignLeft ) );
    inarowLayout->addWidget( TextLabel6_2 );
    Spacer6 = new QSpacerItem( 0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding );
    inarowLayout->addItem( Spacer6 );
    TabWidget2->insertTab( inarow, QString::fromLatin1("") );

    count = new QWidget( TabWidget2, "count" );
    countLayout = new QVBoxLayout( count, 11, 6, "countLayout"); 

    Layout66 = new QHBoxLayout( 0, 0, 6, "Layout66"); 
    Spacer16 = new QSpacerItem( 0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
    Layout66->addItem( Spacer16 );

    Layout65 = new QGridLayout( 0, 1, 1, 0, 6, "Layout65"); 

    countEval2 = new QLineEdit( count, "countEval2" );
    countEval2->setMaximumSize( QSize( 40, 32767 ) );
    countEval2->setAlignment( int( QLineEdit::AlignLeft ) );

    Layout65->addWidget( countEval2, 1, 1 );

    TextLabel1_2_2_3_4_2 = new QLabel( count, "TextLabel1_2_2_3_4_2" );

    Layout65->addWidget( TextLabel1_2_2_3_4_2, 3, 0 );

    countEval5 = new QLineEdit( count, "countEval5" );
    countEval5->setMaximumSize( QSize( 40, 32767 ) );
    countEval5->setAlignment( int( QLineEdit::AlignLeft ) );

    Layout65->addWidget( countEval5, 4, 1 );

    countEval4 = new QLineEdit( count, "countEval4" );
    countEval4->setMaximumSize( QSize( 40, 32767 ) );
    countEval4->setAlignment( int( QLineEdit::AlignLeft ) );

    Layout65->addWidget( countEval4, 3, 1 );

    TextLabel1_2_2_3_3_2 = new QLabel( count, "TextLabel1_2_2_3_3_2" );

    Layout65->addWidget( TextLabel1_2_2_3_3_2, 2, 0 );

    TextLabel1_2_2_3_4_2_2 = new QLabel( count, "TextLabel1_2_2_3_4_2_2" );

    Layout65->addWidget( TextLabel1_2_2_3_4_2_2, 4, 0 );

    countEval1 = new QLineEdit( count, "countEval1" );
    countEval1->setMaximumSize( QSize( 40, 32767 ) );
    countEval1->setAlignment( int( QLineEdit::AlignLeft ) );

    Layout65->addWidget( countEval1, 0, 1 );

    TextLabel1_2_2_3_2_2 = new QLabel( count, "TextLabel1_2_2_3_2_2" );

    Layout65->addWidget( TextLabel1_2_2_3_2_2, 1, 0 );

    countEval3 = new QLineEdit( count, "countEval3" );
    countEval3->setMaximumSize( QSize( 40, 32767 ) );
    countEval3->setAlignment( int( QLineEdit::AlignLeft ) );

    Layout65->addWidget( countEval3, 2, 1 );

    TextLabel1_2_2_3_5 = new QLabel( count, "TextLabel1_2_2_3_5" );

    Layout65->addWidget( TextLabel1_2_2_3_5, 0, 0 );
    Layout66->addLayout( Layout65 );
    Spacer15 = new QSpacerItem( 0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
    Layout66->addItem( Spacer15 );
    countLayout->addLayout( Layout66 );

    TextLabel6_2_2 = new QLabel( count, "TextLabel6_2_2" );
    TextLabel6_2_2->setAlignment( int( QLabel::WordBreak | QLabel::AlignTop | QLabel::AlignLeft ) );
    countLayout->addWidget( TextLabel6_2_2 );
    Spacer6_2 = new QSpacerItem( 0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding );
    countLayout->addItem( Spacer6_2 );
    TabWidget2->insertTab( count, QString::fromLatin1("") );

    tab = new QWidget( TabWidget2, "tab" );
    tabLayout = new QVBoxLayout( tab, 11, 6, "tabLayout"); 

    evalList = new QListBox( tab, "evalList" );
    tabLayout->addWidget( evalList );

    Layout13 = new QHBoxLayout( 0, 0, 6, "Layout13"); 

    evalSaveAs = new QPushButton( tab, "evalSaveAs" );
    Layout13->addWidget( evalSaveAs );

    evalDelete = new QPushButton( tab, "evalDelete" );
    Layout13->addWidget( evalDelete );
    tabLayout->addLayout( Layout13 );

    TextLabel1 = new QLabel( tab, "TextLabel1" );
    TextLabel1->setAlignment( int( QLabel::WordBreak | QLabel::AlignTop | QLabel::AlignLeft ) );
    tabLayout->addWidget( TextLabel1 );
    TabWidget2->insertTab( tab, QString::fromLatin1("") );
    EvalDlgLayout->addWidget( TabWidget2 );

    Layout13_2 = new QHBoxLayout( 0, 0, 6, "Layout13_2"); 
    Spacer2_3 = new QSpacerItem( 0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
    Layout13_2->addItem( Spacer2_3 );

    TextLabel9 = new QLabel( this, "TextLabel9" );
    Layout13_2->addWidget( TextLabel9 );

    actualEval = new QLCDNumber( this, "actualEval" );
    actualEval->setSegmentStyle( QLCDNumber::Flat );
    Layout13_2->addWidget( actualEval );
    Spacer1 = new QSpacerItem( 0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
    Layout13_2->addItem( Spacer1 );
    EvalDlgLayout->addLayout( Layout13_2 );
    languageChange();
    resize( QSize(350, 388).expandedTo(minimumSizeHint()) );
    clearWState( WState_Polished );

    // tab order
    setTabOrder( TabWidget2, moveEval1 );
    setTabOrder( moveEval1, moveEval2 );
    setTabOrder( moveEval2, moveEval3 );
    setTabOrder( moveEval3, moveEval4 );
    setTabOrder( moveEval4, moveEval5 );
    setTabOrder( moveEval5, moveEval6 );
    setTabOrder( moveEval6, moveEval7 );
    setTabOrder( moveEval7, moveEval8 );
    setTabOrder( moveEval8, moveEval9 );
    setTabOrder( moveEval9, posEval1 );
    setTabOrder( posEval1, posEval2 );
    setTabOrder( posEval2, diffEval2 );
    setTabOrder( diffEval2, posEval3 );
    setTabOrder( posEval3, diffEval3 );
    setTabOrder( diffEval3, posEval4 );
    setTabOrder( posEval4, diffEval4 );
    setTabOrder( diffEval4, posEval5 );
    setTabOrder( posEval5, diffEval5 );
    setTabOrder( diffEval5, rowEval2 );
    setTabOrder( rowEval2, rowEval3 );
    setTabOrder( rowEval3, rowEval4 );
    setTabOrder( rowEval4, rowEval5 );
    setTabOrder( rowEval5, countEval1 );
    setTabOrder( countEval1, countEval2 );
    setTabOrder( countEval2, countEval3 );
    setTabOrder( countEval3, countEval4 );
    setTabOrder( countEval4, countEval5 );
    setTabOrder( countEval5, evalList );
    setTabOrder( evalList, evalDelete );
    setTabOrder( evalDelete, evalSaveAs );
}

/*
 *  Destroys the object and frees any allocated resources
 */
EvalDlg::~EvalDlg()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void EvalDlg::languageChange()
{
    setCaption( tr2i18n( "Configure Evaluation" ) );
    TextLabel4_2_2->setText( tr2i18n( "Push Out" ) );
    TextLabel4_2->setText( tr2i18n( "Push" ) );
    TextLabel4->setText( tr2i18n( "Normal" ) );
    TextLabel5->setText( tr2i18n( "For every move possible the given points are added to the Evaluation." ) );
    TabWidget2->changeTab( moves, tr2i18n( "Moves" ) );
    TextLabel1_2_3_2->setText( tr2i18n( "Inner ring 3:" ) );
    TextLabel1_2->setText( tr2i18n( "Outermost ring:" ) );
    TextLabel1_2_2->setText( tr2i18n( "Middle position:" ) );
    TextLabel1_2_3->setText( tr2i18n( "Inner ring 2:" ) );
    TextLabel3_4->setText( tr2i18n( "+/-" ) );
    TextLabel3_3->setText( tr2i18n( "+/-" ) );
    TextLabel3_2->setText( tr2i18n( "+/-" ) );
    TextLabel1_2_2_2->setText( tr2i18n( "Innermost ring:" ) );
    TextLabel3->setText( tr2i18n( "+/-" ) );
    TextLabel6->setText( tr2i18n( "For every ball, the given points are added to the evaluation depending on the balls position. The bonus for a given position is changed randomly in the +/- range." ) );
    TabWidget2->changeTab( position, tr2i18n( "Position" ) );
    TextLabel1_2_2_3_2->setText( tr2i18n( "Three in-a-row:" ) );
    TextLabel1_2_2_3->setText( tr2i18n( "Two in-a-row:" ) );
    TextLabel1_2_2_3_3->setText( tr2i18n( "Four in-a-row:" ) );
    TextLabel1_2_2_3_4->setText( tr2i18n( "Five in-a-row:" ) );
    TextLabel6_2->setText( tr2i18n( "For a number of balls In-a-Row, the given points are added to the evaluation" ) );
    TabWidget2->changeTab( inarow, tr2i18n( "In-A-Row" ) );
    TextLabel1_2_2_3_4_2->setText( tr2i18n( "4 Balls more:" ) );
    TextLabel1_2_2_3_3_2->setText( tr2i18n( "3 Balls more:" ) );
    TextLabel1_2_2_3_4_2_2->setText( tr2i18n( "5 Balls more:" ) );
    TextLabel1_2_2_3_2_2->setText( tr2i18n( "2 Balls more:" ) );
    TextLabel1_2_2_3_5->setText( tr2i18n( "1 Ball more:" ) );
    TextLabel6_2_2->setText( tr2i18n( "For a difference in the number of balls, the given points are added to the evaluation. A difference of 6 only can be a lost/won game." ) );
    TabWidget2->changeTab( count, tr2i18n( "Count" ) );
    evalSaveAs->setText( tr2i18n( "Save As..." ) );
    evalDelete->setText( tr2i18n( "Delete" ) );
    TextLabel1->setText( tr2i18n( "Your evaluation scheme, defined in all other tabs of this dialog, can be stored here." ) );
    TabWidget2->changeTab( tab, tr2i18n( "Evaluation Schemes" ) );
    TextLabel9->setText( tr2i18n( "Evaluation of actual position:" ) );
}

#include "EvalDlg.moc"
