/****************************************************************************
** Form interface generated from reading ui file '/usr/src/ports/kdegames3/kdegames3-3.5.10-11.x86_64/src/kdegames-3.5.10/kenolaba/EvalDlg.ui'
**
** Created by User Interface Compiler
**
** WARNING! All changes made in this file will be lost!
****************************************************************************/

#ifndef EVALDLG_H
#define EVALDLG_H

#include <qvariant.h>
#include <qpixmap.h>
#include <qwidget.h>
#include <kseparator.h>

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QSpacerItem;
class QTabWidget;
class QLineEdit;
class QLabel;
class QListBox;
class QListBoxItem;
class QPushButton;
class QLCDNumber;

class EvalDlg : public QWidget
{
    Q_OBJECT

public:
    EvalDlg( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~EvalDlg();

    QTabWidget* TabWidget2;
    QWidget* moves;
    QLineEdit* moveEval1;
    QLabel* PixmapLabel3_2;
    QLineEdit* moveEval2;
    QLabel* PixmapLabel1_2_5;
    QLineEdit* moveEval3;
    QLabel* PixmapLabel3_3;
    QLabel* unnamed;
    QLabel* PixmapLabel1_2_6;
    QLabel* PixmapLabel1_2_4;
    QLabel* PixmapLabel3;
    QLabel* PixmapLabel1_2;
    QLabel* PixmapLabel1_3;
    QLabel* TextLabel4_2_2;
    QLabel* TextLabel4_2;
    QLabel* PixmapLabel1_2_5_2_3;
    QLabel* PixmapLabel3_3_2_3;
    QLabel* PixmapLabel3_3_2_2;
    QLineEdit* moveEval4;
    QLineEdit* moveEval5;
    QLabel* PixmapLabel2_3;
    QLabel* PixmapLabel1_2_5_2;
    QLabel* PixmapLabel3_3_2;
    QLabel* PixmapLabel2_2_2;
    QLabel* PixmapLabel1_3_2;
    QLineEdit* moveEval6;
    QLabel* PixmapLabel1_4;
    QLabel* PixmapLabel1_2_2;
    QLabel* PixmapLabel2_2;
    QLabel* PixmapLabel1_4_3;
    QLabel* PixmapLabel1_2_5_2_2;
    QLabel* PixmapLabel2;
    QLabel* PixmapLabel1_4_2;
    QLabel* PixmapLabel2_4;
    QLabel* PixmapLabel1_3_2_2;
    QLabel* PixmapLabel1_2_5_2_2_2;
    QLineEdit* moveEval7;
    QLabel* PixmapLabel1_2_2_2;
    QLabel* PixmapLabel2_2_2_3;
    QLineEdit* moveEval9;
    QLabel* PixmapLabel1_4_2_2;
    QLabel* PixmapLabel1_2_5_2_3_3;
    QLineEdit* moveEval8;
    QLabel* PixmapLabel1_4_3_2;
    QLabel* PixmapLabel1_4_4;
    QLabel* PixmapLabel1_2_5_2_4;
    QLabel* PixmapLabel2_3_3;
    QLabel* PixmapLabel2_2_3;
    QLabel* TextLabel4;
    QLabel* TextLabel5;
    QWidget* position;
    QLabel* TextLabel1_2_3_2;
    QLineEdit* posEval3;
    QLineEdit* posEval5;
    QLineEdit* posEval2;
    QLabel* TextLabel1_2;
    QLineEdit* diffEval2;
    QLineEdit* diffEval3;
    QLineEdit* posEval4;
    QLabel* TextLabel1_2_2;
    QLineEdit* diffEval5;
    QLineEdit* posEval1;
    QLabel* TextLabel1_2_3;
    QLabel* TextLabel3_4;
    QLabel* TextLabel3_3;
    QLineEdit* diffEval4;
    QLabel* TextLabel3_2;
    QLabel* TextLabel1_2_2_2;
    QLabel* TextLabel3;
    QLabel* TextLabel6;
    QWidget* inarow;
    QLineEdit* rowEval3;
    QLineEdit* rowEval2;
    QLineEdit* rowEval5;
    QLineEdit* rowEval4;
    QLabel* TextLabel1_2_2_3_2;
    QLabel* TextLabel1_2_2_3;
    QLabel* TextLabel1_2_2_3_3;
    QLabel* TextLabel1_2_2_3_4;
    QLabel* TextLabel6_2;
    QWidget* count;
    QLineEdit* countEval2;
    QLabel* TextLabel1_2_2_3_4_2;
    QLineEdit* countEval5;
    QLineEdit* countEval4;
    QLabel* TextLabel1_2_2_3_3_2;
    QLabel* TextLabel1_2_2_3_4_2_2;
    QLineEdit* countEval1;
    QLabel* TextLabel1_2_2_3_2_2;
    QLineEdit* countEval3;
    QLabel* TextLabel1_2_2_3_5;
    QLabel* TextLabel6_2_2;
    QWidget* tab;
    QListBox* evalList;
    QPushButton* evalSaveAs;
    QPushButton* evalDelete;
    QLabel* TextLabel1;
    QLabel* TextLabel9;
    QLCDNumber* actualEval;

protected:
    QVBoxLayout* EvalDlgLayout;
    QVBoxLayout* movesLayout;
    QSpacerItem* Spacer5;
    QGridLayout* Layout17;
    QSpacerItem* Spacer9_2;
    QSpacerItem* Spacer9;
    QGridLayout* Layout35;
    QSpacerItem* Spacer2;
    QGridLayout* Layout37;
    QSpacerItem* Spacer2_2;
    QGridLayout* Layout38;
    QSpacerItem* Spacer2_2_2;
    QVBoxLayout* positionLayout;
    QSpacerItem* Spacer4;
    QHBoxLayout* Layout63;
    QSpacerItem* Spacer12;
    QSpacerItem* Spacer11;
    QGridLayout* Layout62;
    QVBoxLayout* inarowLayout;
    QSpacerItem* Spacer6;
    QHBoxLayout* Layout64;
    QSpacerItem* Spacer14;
    QSpacerItem* Spacer13;
    QGridLayout* Layout42;
    QVBoxLayout* countLayout;
    QSpacerItem* Spacer6_2;
    QHBoxLayout* Layout66;
    QSpacerItem* Spacer16;
    QSpacerItem* Spacer15;
    QGridLayout* Layout65;
    QVBoxLayout* tabLayout;
    QHBoxLayout* Layout13;
    QHBoxLayout* Layout13_2;
    QSpacerItem* Spacer2_3;
    QSpacerItem* Spacer1;

protected slots:
    virtual void languageChange();

private:
    QPixmap image0;
    QPixmap image1;
    QPixmap image2;

};

#endif // EVALDLG_H
