// This file is generated by kconfig_compiler from libksirtet1.kcfg.
// All changes you do to this file will be lost.

#include "baseprefs.h"

#include <kstaticdeleter.h>

BasePrefs *BasePrefs::mSelf = 0;
static KStaticDeleter<BasePrefs> staticBasePrefsDeleter;

BasePrefs *BasePrefs::self()
{
  if ( !mSelf ) {
    staticBasePrefsDeleter.setObject( mSelf, new BasePrefs() );
    mSelf->readConfig();
  }

  return mSelf;
}

BasePrefs::BasePrefs(  )
  : KConfigSkeleton()
{
  mSelf = this;
  setCurrentGroup( QString::fromLatin1( "Options" ) );

  KConfigSkeleton::ItemInt  *itemBlockSize;
  itemBlockSize = new KConfigSkeleton::ItemInt( currentGroup(), QString::fromLatin1( "block size" ), mBlockSize, 15 );
  itemBlockSize->setMinValue(4);
  itemBlockSize->setMaxValue(100);
  addItem( itemBlockSize, QString::fromLatin1( "BlockSize" ) );
  KConfigSkeleton::ItemDouble  *itemFadeIntensity;
  itemFadeIntensity = new KConfigSkeleton::ItemDouble( currentGroup(), QString::fromLatin1( "fade intensity" ), mFadeIntensity, 1.0 );
  addItem( itemFadeIntensity, QString::fromLatin1( "FadeIntensity" ) );
  KConfigSkeleton::ItemColor  *itemFadeColor;
  itemFadeColor = new KConfigSkeleton::ItemColor( currentGroup(), QString::fromLatin1( "fade color" ), mFadeColor, QColor( "black" ) );
  addItem( itemFadeColor, QString::fromLatin1( "FadeColor" ) );
  KConfigSkeleton::ItemBool  *itemAnimationsEnabled;
  itemAnimationsEnabled = new KConfigSkeleton::ItemBool( currentGroup(), QString::fromLatin1( "enable animations" ), mAnimationsEnabled, true );
  addItem( itemAnimationsEnabled, QString::fromLatin1( "AnimationsEnabled" ) );
  KConfigSkeleton::ItemBool  *itemMenubarVisible;
  itemMenubarVisible = new KConfigSkeleton::ItemBool( currentGroup(), QString::fromLatin1( "menubar visible" ), mMenubarVisible, true );
  addItem( itemMenubarVisible, QString::fromLatin1( "MenubarVisible" ) );
  KConfigSkeleton::ItemColor  *itemColor[11];
  itemColor[0] = new KConfigSkeleton::ItemColor( currentGroup(), QString::fromLatin1( "color #0" ), mColor[0], Piece::info().defaultColor(0) );
  addItem( itemColor[0], QString::fromLatin1( "Color0" ) );
  itemColor[1] = new KConfigSkeleton::ItemColor( currentGroup(), QString::fromLatin1( "color #1" ), mColor[1], Piece::info().defaultColor(1) );
  addItem( itemColor[1], QString::fromLatin1( "Color1" ) );
  itemColor[2] = new KConfigSkeleton::ItemColor( currentGroup(), QString::fromLatin1( "color #2" ), mColor[2], Piece::info().defaultColor(2) );
  addItem( itemColor[2], QString::fromLatin1( "Color2" ) );
  itemColor[3] = new KConfigSkeleton::ItemColor( currentGroup(), QString::fromLatin1( "color #3" ), mColor[3], Piece::info().defaultColor(3) );
  addItem( itemColor[3], QString::fromLatin1( "Color3" ) );
  itemColor[4] = new KConfigSkeleton::ItemColor( currentGroup(), QString::fromLatin1( "color #4" ), mColor[4], Piece::info().defaultColor(4) );
  addItem( itemColor[4], QString::fromLatin1( "Color4" ) );
  itemColor[5] = new KConfigSkeleton::ItemColor( currentGroup(), QString::fromLatin1( "color #5" ), mColor[5], Piece::info().defaultColor(5) );
  addItem( itemColor[5], QString::fromLatin1( "Color5" ) );
  itemColor[6] = new KConfigSkeleton::ItemColor( currentGroup(), QString::fromLatin1( "color #6" ), mColor[6], Piece::info().defaultColor(6) );
  addItem( itemColor[6], QString::fromLatin1( "Color6" ) );
  itemColor[7] = new KConfigSkeleton::ItemColor( currentGroup(), QString::fromLatin1( "color #7" ), mColor[7], Piece::info().defaultColor(7) );
  addItem( itemColor[7], QString::fromLatin1( "Color7" ) );
  itemColor[8] = new KConfigSkeleton::ItemColor( currentGroup(), QString::fromLatin1( "color #8" ), mColor[8], Piece::info().defaultColor(8) );
  addItem( itemColor[8], QString::fromLatin1( "Color8" ) );
  itemColor[9] = new KConfigSkeleton::ItemColor( currentGroup(), QString::fromLatin1( "color #9" ), mColor[9], Piece::info().defaultColor(9) );
  addItem( itemColor[9], QString::fromLatin1( "Color9" ) );
  itemColor[10] = new KConfigSkeleton::ItemColor( currentGroup(), QString::fromLatin1( "color #10" ), mColor[10], Piece::info().defaultColor(10) );
  addItem( itemColor[10], QString::fromLatin1( "Color10" ) );
}

BasePrefs::~BasePrefs()
{
  if ( mSelf == this )
    staticBasePrefsDeleter.setObject( mSelf, 0, false );
}

