// This file is generated by kconfig_compiler from ksirtet.kcfg.
// All changes you do to this file will be lost.

#include "prefs.h"

#include <kstaticdeleter.h>

Prefs *Prefs::mSelf = 0;
static KStaticDeleter<Prefs> staticPrefsDeleter;

Prefs *Prefs::self()
{
  if ( !mSelf ) {
    staticPrefsDeleter.setObject( mSelf, new Prefs() );
    mSelf->readConfig();
  }

  return mSelf;
}

Prefs::Prefs(  )
  : CommonPrefs()
{
  mSelf = this;
  setCurrentGroup( QString::fromLatin1( "Options" ) );

  KConfigSkeleton::ItemBool  *itemOldRotationStyle;
  itemOldRotationStyle = new KConfigSkeleton::ItemBool( currentGroup(), QString::fromLatin1( "old rotation style" ), mOldRotationStyle, false );
  addItem( itemOldRotationStyle, QString::fromLatin1( "OldRotationStyle" ) );

  setCurrentGroup( QString::fromLatin1( "AI" ) );

  KConfigSkeleton::ItemDouble  *itemCoefficient_OccupiedLines;
  itemCoefficient_OccupiedLines = new KConfigSkeleton::ItemDouble( currentGroup(), QString::fromLatin1( "occupied lines COEF" ), mCoefficient_OccupiedLines, -2.0 );
  itemCoefficient_OccupiedLines->setMinValue(-100.0);
  itemCoefficient_OccupiedLines->setMaxValue(0.0);
  addItem( itemCoefficient_OccupiedLines, QString::fromLatin1( "Coefficient_OccupiedLines" ) );
  KConfigSkeleton::ItemDouble  *itemCoefficient_Holes;
  itemCoefficient_Holes = new KConfigSkeleton::ItemDouble( currentGroup(), QString::fromLatin1( "holes COEF" ), mCoefficient_Holes, -4.0 );
  itemCoefficient_Holes->setMinValue(-100.0);
  itemCoefficient_Holes->setMaxValue(0.0);
  addItem( itemCoefficient_Holes, QString::fromLatin1( "Coefficient_Holes" ) );
  KConfigSkeleton::ItemDouble  *itemCoefficient_Spaces;
  itemCoefficient_Spaces = new KConfigSkeleton::ItemDouble( currentGroup(), QString::fromLatin1( "spaces COEF" ), mCoefficient_Spaces, -2.0 );
  itemCoefficient_Spaces->setMinValue(-100.0);
  itemCoefficient_Spaces->setMaxValue(0.0);
  addItem( itemCoefficient_Spaces, QString::fromLatin1( "Coefficient_Spaces" ) );
  KConfigSkeleton::ItemDouble  *itemCoefficient_Peak2Peak;
  itemCoefficient_Peak2Peak = new KConfigSkeleton::ItemDouble( currentGroup(), QString::fromLatin1( "peak-to-peak COEF" ), mCoefficient_Peak2Peak, -2.0 );
  itemCoefficient_Peak2Peak->setMinValue(-100.0);
  itemCoefficient_Peak2Peak->setMaxValue(0.0);
  addItem( itemCoefficient_Peak2Peak, QString::fromLatin1( "Coefficient_Peak2Peak" ) );
  KConfigSkeleton::ItemDouble  *itemCoefficient_MeanHeight;
  itemCoefficient_MeanHeight = new KConfigSkeleton::ItemDouble( currentGroup(), QString::fromLatin1( "mean height COEF" ), mCoefficient_MeanHeight, 0.0 );
  itemCoefficient_MeanHeight->setMinValue(-100.0);
  itemCoefficient_MeanHeight->setMaxValue(0.0);
  addItem( itemCoefficient_MeanHeight, QString::fromLatin1( "Coefficient_MeanHeight" ) );
  KConfigSkeleton::ItemDouble  *itemCoefficient_FullLines;
  itemCoefficient_FullLines = new KConfigSkeleton::ItemDouble( currentGroup(), QString::fromLatin1( "full lines COEF" ), mCoefficient_FullLines, 50.0 );
  itemCoefficient_FullLines->setMinValue(-0.0);
  itemCoefficient_FullLines->setMaxValue(100.0);
  addItem( itemCoefficient_FullLines, QString::fromLatin1( "Coefficient_FullLines" ) );
  KConfigSkeleton::ItemInt  *itemTrigger_FullLines;
  itemTrigger_FullLines = new KConfigSkeleton::ItemInt( currentGroup(), QString::fromLatin1( "full lines TRIG" ), mTrigger_FullLines, 1 );
  itemTrigger_FullLines->setMinValue(1);
  itemTrigger_FullLines->setMaxValue(4);
  addItem( itemTrigger_FullLines, QString::fromLatin1( "Trigger_FullLines" ) );
}

Prefs::~Prefs()
{
  if ( mSelf == this )
    staticPrefsDeleter.setObject( mSelf, 0, false );
}

