// This file is generated by kconfig_compiler from kfouleggs.kcfg.
// All changes you do to this file will be lost.

#include "prefs.h"

#include <kstaticdeleter.h>

Prefs *Prefs::mSelf = 0;
static KStaticDeleter<Prefs> staticPrefsDeleter;

Prefs *Prefs::self()
{
  if ( !mSelf ) {
    staticPrefsDeleter.setObject( mSelf, new Prefs() );
    mSelf->readConfig();
  }

  return mSelf;
}

Prefs::Prefs(  )
  : CommonPrefs()
{
  mSelf = this;
  setCurrentGroup( QString::fromLatin1( "AI" ) );

  KConfigSkeleton::ItemDouble  *itemCoefficient_OccupiedLines;
  itemCoefficient_OccupiedLines = new KConfigSkeleton::ItemDouble( currentGroup(), QString::fromLatin1( "occupied lines COEF" ), mCoefficient_OccupiedLines, -2.0 );
  itemCoefficient_OccupiedLines->setMinValue(-100.0);
  itemCoefficient_OccupiedLines->setMaxValue(0.0);
  addItem( itemCoefficient_OccupiedLines, QString::fromLatin1( "Coefficient_OccupiedLines" ) );
  KConfigSkeleton::ItemDouble  *itemCoefficient_Spaces;
  itemCoefficient_Spaces = new KConfigSkeleton::ItemDouble( currentGroup(), QString::fromLatin1( "spaces COEF" ), mCoefficient_Spaces, 0.0 );
  itemCoefficient_Spaces->setMinValue(-100.0);
  itemCoefficient_Spaces->setMaxValue(0.0);
  addItem( itemCoefficient_Spaces, QString::fromLatin1( "Coefficient_Spaces" ) );
  KConfigSkeleton::ItemDouble  *itemCoefficient_Peak2Peak;
  itemCoefficient_Peak2Peak = new KConfigSkeleton::ItemDouble( currentGroup(), QString::fromLatin1( "peak-to-peak COEF" ), mCoefficient_Peak2Peak, 0.0 );
  itemCoefficient_Peak2Peak->setMinValue(-100.0);
  itemCoefficient_Peak2Peak->setMaxValue(0.0);
  addItem( itemCoefficient_Peak2Peak, QString::fromLatin1( "Coefficient_Peak2Peak" ) );
  KConfigSkeleton::ItemDouble  *itemCoefficient_MeanHeight;
  itemCoefficient_MeanHeight = new KConfigSkeleton::ItemDouble( currentGroup(), QString::fromLatin1( "mean height COEF" ), mCoefficient_MeanHeight, 0.0 );
  itemCoefficient_MeanHeight->setMinValue(-100.0);
  itemCoefficient_MeanHeight->setMaxValue(0.0);
  addItem( itemCoefficient_MeanHeight, QString::fromLatin1( "Coefficient_MeanHeight" ) );
  KConfigSkeleton::ItemDouble  *itemCoefficient_RemovedEggs;
  itemCoefficient_RemovedEggs = new KConfigSkeleton::ItemDouble( currentGroup(), QString::fromLatin1( "removed eggs COEF" ), mCoefficient_RemovedEggs, 20.0 );
  itemCoefficient_RemovedEggs->setMinValue(0.0);
  itemCoefficient_RemovedEggs->setMaxValue(100.0);
  addItem( itemCoefficient_RemovedEggs, QString::fromLatin1( "Coefficient_RemovedEggs" ) );
  KConfigSkeleton::ItemDouble  *itemCoefficient_Puyos;
  itemCoefficient_Puyos = new KConfigSkeleton::ItemDouble( currentGroup(), QString::fromLatin1( "nb puyos COEF" ), mCoefficient_Puyos, 50.0 );
  itemCoefficient_Puyos->setMinValue(0.0);
  itemCoefficient_Puyos->setMaxValue(100.0);
  addItem( itemCoefficient_Puyos, QString::fromLatin1( "Coefficient_Puyos" ) );
  KConfigSkeleton::ItemInt  *itemTrigger_Puyos;
  itemTrigger_Puyos = new KConfigSkeleton::ItemInt( currentGroup(), QString::fromLatin1( "nb puyos TRIG" ), mTrigger_Puyos, 0 );
  itemTrigger_Puyos->setMinValue(0);
  itemTrigger_Puyos->setMaxValue(10);
  addItem( itemTrigger_Puyos, QString::fromLatin1( "Trigger_Puyos" ) );
  KConfigSkeleton::ItemDouble  *itemCoefficient_ChainedPuyos;
  itemCoefficient_ChainedPuyos = new KConfigSkeleton::ItemDouble( currentGroup(), QString::fromLatin1( "nb chained COEF" ), mCoefficient_ChainedPuyos, 80.0 );
  itemCoefficient_ChainedPuyos->setMinValue(0.0);
  itemCoefficient_ChainedPuyos->setMaxValue(100.0);
  addItem( itemCoefficient_ChainedPuyos, QString::fromLatin1( "Coefficient_ChainedPuyos" ) );
  KConfigSkeleton::ItemInt  *itemTrigger_ChainedPuyos;
  itemTrigger_ChainedPuyos = new KConfigSkeleton::ItemInt( currentGroup(), QString::fromLatin1( "nb chained TRIG" ), mTrigger_ChainedPuyos, 0 );
  itemTrigger_ChainedPuyos->setMinValue(0);
  itemTrigger_ChainedPuyos->setMaxValue(10);
  addItem( itemTrigger_ChainedPuyos, QString::fromLatin1( "Trigger_ChainedPuyos" ) );
}

Prefs::~Prefs()
{
  if ( mSelf == this )
    staticPrefsDeleter.setObject( mSelf, 0, false );
}

